const channelsID = require('../channels_conf.json');
const complotDB = require('./ComplotDB.js');
const {Category} = require('./Category.js');

class BaseInfo{
    constructor(client){
        this.client = client;
        this.#resetStoredMessages();
        this.channelBase = this.client.channels.cache.get(channelsID.base);
    }

    #resetStoredMessages() {
        this.infoMessagesActor = [];
        this.infoMessagesReason = [];
        this.infoMessagesAction = [];
        this.infoMessagesSource = [];
        this.infoMessagesSentence = [];
    }

    getInfoMessageFromCategory(category){
        switch(category){
            case Category.ACTION : return this.infoMessagesAction;
            case Category.REASON : return this.infoMessagesReason;
            case Category.SENTENCE : return this.infoMessagesSentence;
            case Category.SOURCE : return this.infoMessagesSource;
            case Category.ACTOR : return this.infoMessagesActor;
        }
        return null;
    }


    async initAllBase(){
        const cats = [
            Category.ACTOR, Category.ACTION, Category.REASON, Category.SOURCE, Category.SENTENCE
        ];

        await this.deleteAllMessageInfo();

        for(let category of cats){
            const splitMessages = complotDB.categoryToSplitMessage(category).map(el => '```' + el + '```');
            for(let messagePart of splitMessages){
                this.getInfoMessageFromCategory(category).push(await this.channelBase.send(messagePart));
            }
        }
    }

    async updateBase(category){   
        if (!(this.infoMessagesActor instanceof Array) || this.infoMessagesActor.length === 0) {
            this.initAllBase();
        }else{
            const infoMessages = this.getInfoMessageFromCategory(category);
            const splitMessages = complotDB.categoryToSplitMessage(category).map(el => '```' + el + '```');
            // If we had more messages before the update, we get rid of them here
            infoMessages.splice(splitMessages.length)
            // If in the opposite we have more messages now, we re-send everything
            if (splitMessages.length > infoMessages.length) {
                this.#resetStoredMessages();
                return await this.updateBase(category);
            }
            // Now we edit!
            for (let i = 0; i < splitMessages.length; i++)
                infoMessages[i].edit(splitMessages[i]);
        }
    }

    async deleteAllMessageInfo(){
        let messages = await this.channelBase.messages.fetch();
        let workers = [];
        messages.forEach(m=>{
            workers.push(m.delete());
        });
        try{
            await Promise.all(workers);
        }catch(e){
            console.error(e);
        }
    }

}

module.exports = BaseInfo;
