
const complotDB = require('./ComplotDB.js');
const { Category } = require('./Category.js');

const actorRE = new RegExp('<actor>', 'gi');
const wantRE = new RegExp('<want>', 'gi');
const reasonRE = new RegExp('<reason>', 'gi');
const sourceRE = new RegExp('<source>', 'gi');
const actionRE = new RegExp('<action>', 'gi');

const MAX_ITERATIONS = 10;

function fillAllRandomObject(sentence, regularExpr, category){

    const matches = sentence.matchAll(regularExpr);
    let objectString = "";
    let newSentence = sentence;
    for (let match of matches) {
        objectString = complotDB.getRandom(category);
        newSentence = newSentence.replace(match, objectString);
    }

    return newSentence;
}

function shouldBeFilled(sentence) {

    return [actorRE, reasonRE, sourceRE, actionRE].some((re) =>{
        const result = re.test(sentence)
        re.lastIndex = 0;
        return result; 
    });
}

function getRandomSentenceFilled(sentence){
    let it = 0;
    while(shouldBeFilled(sentence)){
        sentence = fillAllRandomObject(sentence, actorRE, Category.ACTOR);
        sentence = fillAllRandomObject(sentence, actionRE, Category.ACTION);
        sentence = fillAllRandomObject(sentence, reasonRE, Category.REASON);
        sentence = fillAllRandomObject(sentence, sourceRE, Category.SOURCE);

        it++;
        if(it >= MAX_ITERATIONS) break;
    }
    

    sentence = sentence.replaceAll(wantRE, 'veut');
    return sentence;
}

function getRandomComplot(){
    return getRandomSentenceFilled(complotDB.getRandom(Category.SENTENCE));
}

function getRandomActor(){
    return getRandomSentenceFilled(complotDB.getRandom(Category.ACTOR));
}

function getRandomReason(){
    return getRandomSentenceFilled(complotDB.getRandom(Category.REASON));
}

function getRandomWelcome(){
    return getRandomSentenceFilled(complotDB.getRandom(Category.WELCOME));
}


module.exports = {
    getRandomSentenceFilled,
    getRandomComplot,
    getRandomActor,
    getRandomReason,
    getRandomWelcome
}